<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="kig_shell" version="4">
<MenuBar>
	<Menu name="file" noMerge="1"><text>&amp;File</text>
		<Action name="file_new"/>
		<Action name="file_open"/>
		<Action name="file_open_recent"/>
		<Separator />
		<Merge />
		<Action name="file_quit"/>
	</Menu>
	<Merge />
	<Menu name="settings" noMerge="1"><text>&amp;Settings</text>
		<Action name="options_show_toolbar"/>
		<Merge name="StandardToolBarMenuHandler" />
		<Action name="options_show_statusbar"/>
                <Merge />
		<Separator/>
		<Action name="options_configure_keybinding"/>
		<Action name="options_configure_toolbars"/>
	</Menu>
</MenuBar>
<ToolBar name="mainToolBar" noMerge="1"><text>Main Toolbar</text>
  <Action name="file_new"/>
  <Action name="file_open"/>
  <Separator/>
  <Merge />
</ToolBar>
</kpartgui>
