/*
 This file is part of Kig, a KDE program for Interactive Geometry...
 Copyright (C) 2006  Pino Toscano <toscano.pino@tiscali.it>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 USA
*/

#ifndef KIG_MODES_HISTORYDIALOG_H
#define KIG_MODES_HISTORYDIALOG_H

class QUndoStack;
class QWidget;
class Ui_HistoryWidget;

#include <QDialog>

/**
 * The HistoryDialog represents a small dialog to navigate back and forth in
 * the history of a construction.
 */
class HistoryDialog
  : public QDialog
{
Q_OBJECT

public:
  HistoryDialog( QUndoStack* kch, QWidget* parent );
  virtual ~HistoryDialog();

private slots:
  void updateWidgets();

  void goToFirst();
  void goBack();
  void goToNext();
  void goToLast();

private:
  QUndoStack* mch;

  Ui_HistoryWidget* mwidget;

  int mtotalsteps;
};

#endif
